import {Component, OnDestroy, OnInit} from '@angular/core';
import {CloudData, CloudOptions} from 'angular-tag-cloud-module';
import {LecturerInfoService} from 'src/app/services/lecturer-info.service';
import {LecturerDataService} from 'src/app/services/lecturer-data.service';
import {WordDetailsDao} from '../../dao/word-details.dao';
import {QuestionDao} from '../../dao/question.dao';

@Component({
  selector:  'app-sentiment-cloud',
  templateUrl: './sentiment-cloud.component.html',
  styleUrls: ['./sentiment-cloud.component.css']
})
export class SentimentCloudComponent implements OnInit, OnDestroy {

  private loading: boolean;
  private maxOccurrences: number;
  private wordClouds: any[][];
  public options: CloudOptions;
  public questions: QuestionDao[];
  public wordcloudData: CloudData[];


  constructor(private lecturerInfoService: LecturerInfoService,
              private lecturerDataService: LecturerDataService) {
  }

  ngOnInit(): void {
    this.questions = [];
    this.wordClouds = [];
    this.wordcloudData = [];
    this.maxOccurrences = 0;
    this.loading = true;
    this.options = {
      width : 1000,
      height : 400,
      overflow: false,
    };
    this.lecturerInfoService.getPaperQuestions(this.lecturerDataService.currentPaper.paperId)
      .subscribe((data: QuestionDao[]) => this.questions = data);
    this.lecturerInfoService.getSentimentCloud(this.lecturerDataService.currentPaper.paperId)
      .subscribe((data: WordDetailsDao[]) => {
      this.sortData(data);
      this.wordcloudData = this.createCloud(this.wordClouds[0]);
      this.loading = false;
    });
  }

  ngOnDestroy(): void {
  }

  private sortData(data: WordDetailsDao[]) {
    this.questions.forEach(question => {
      this.wordClouds.push(data.filter(word => word.questionId === parseInt(question.questionId.toString(), 10)));
    });
  }

  private createCloud(data: WordDetailsDao[]): any {
    const words: any = [];
    data.forEach(word => {
      if (word.numberOfOccurrences > this.maxOccurrences) {
        this.maxOccurrences = word.numberOfOccurrences;
      }
    });
    this.maxOccurrences = this.maxOccurrences / this.questions.length;
    data.forEach(word => {
        words.push({text: word.word, weight: this.getWeight(word.numberOfOccurrences), color: this.getColour(word.polarity) });
    });
    return words;
  }

  private getColour(polarity: number): String {
    switch (polarity) {
      case 0: return '#FF0000';
      case 1: return '#CA6F1E';
      case 2: return '#000000';
      case 3: return '#37ff48';
      case 4: return '#009b13';
      default: return '#000000';
    }
  }

  private getWeight(occurrences: number): number {
    if (occurrences <= this.maxOccurrences / 10) {
      return 2;
    }

    if (occurrences > this.maxOccurrences / 10 && occurrences < this.maxOccurrences / 5) {
      return 4;
    }

    if (occurrences > this.maxOccurrences / 5 && occurrences < this.maxOccurrences / 4) {
      return 6;
    }

    if (occurrences > this.maxOccurrences / 4 && occurrences < this.maxOccurrences / 2) {
      return 8;
    }

    if (occurrences === this.maxOccurrences) {
      return 10;
    }

    return 0;
  }

}
